import Vue from 'vue'
import Vuex from 'vuex'
import axios from 'axios'

Vue.use(Vuex);

export default new Vuex.Store({
	state: {
		count: 0,
		message: 'Get out me yard ya wankers',
		token: null,
		username: null,
		nightMode: false,
	},
	mutations: {
		increment (state) {
			state.count++
		},
		setLoginToken(state, userData){
			
			const username = userData.username
			const token = userData.token

			localStorage.removeItem('loginToken') //We only want one login token per computer
			localStorage.setItem('loginToken', token)

			localStorage.removeItem('username') //We only want one login token per computer
			localStorage.setItem('username', username)

			//Set default token to axios, every request will have header
			axios.defaults.headers.common['Authorization'] = token

			state.token = token
			state.username = username
		},
		destroyLoginToken(state){

			//Remove login token from local storage and from headers
			localStorage.removeItem('loginToken')
			localStorage.removeItem('username')
			delete axios.defaults.headers.common['Authorization']
			state.token = null
			state.username = null
		},
		toggleNightMode(state){

			//Toggle state and save to local storage
			state.nightMode = !(state.nightMode)
			localStorage.setItem('nightMode', state.nightMode)

			//Default theme colors
			let themeColors = {
				'background_color': '#fff',
				'text_color': '#3d3d3d',
				'outline_color': 'rgba(34,36,38,.15)',
			}
			//Night mode colors
			if(state.nightMode){
				themeColors = {
					'background_color': '#000',
					'text_color': '#a98457',
					'outline_color': '#a98457',
				}
			}

			//Go through each color and set CSS variable
			let root = document.documentElement
			Object.keys(themeColors).forEach( attribute => {
				root.style.setProperty('--'+attribute, themeColors[attribute])
			})
		}
	},
	getters: {
		getRudeMessage: state => {
			return state.message
		},
		getUsername: state => {
			return state.username
		},
		getLoginToken: state => {
			return state.token
		},
		getLoggedIn: state => {
			let weIn = (state.token !== null && state.token != undefined && state.token.length > 0)
			return weIn
		},
		getIsNightMode: state => {
			return state.nightMode
		},
	}
})